-- Dump a list of JSON objects representing PE 3.3 Dashboard Classifier Nodes
WITH

  -- Returns an array of tuples crudely expressed using the Postgres "row"
  -- type. This will get cast to JSON as: [{"f1": "key1", "f2": "value1"}, ...]
  dashboard_node_variables AS (
    SELECT
      array_agg(row(key,value)) AS variables,
      parameterable_id
    FROM
      parameters
    WHERE
      parameterable_type = 'Node'
    GROUP BY
      parameterable_id
  ),

  -- Returns an array of parameter tuples for a specific class.
  dashboard_node_class_parameters AS (
    SELECT
      array_agg(row(key,value)) AS parameters,
      parameterable_id
    FROM
      parameters
    WHERE
      parameterable_type = 'NodeClassMembership'
    GROUP BY
      parameterable_id
  ),

  -- Returns an array of (class name, class parameter) tuples for all classes
  -- associated with a node.
  dashboard_node_classes AS (
    SELECT
      array_agg(row(class.name, class.parameters)) AS "classes",
      class.node_id
    FROM (
      SELECT
        node_classes.name,
        COALESCE(dashboard_node_class_parameters.parameters, '{}') as parameters,
        node_class_memberships.node_id
      FROM
        node_class_memberships
      JOIN
        node_classes ON
        node_classes.id = node_class_memberships.node_class_id
      -- Use a LEFT OUTER JOIN to pick up classes that were declared without
      -- parameters.
      LEFT OUTER JOIN
        dashboard_node_class_parameters ON
        node_class_memberships.id = dashboard_node_class_parameters.parameterable_id
    ) AS "class"
    GROUP BY
      class.node_id
  )

SELECT
  row_to_json(dashboard_nodes)
FROM (
  SELECT
    -- Groups created from Console Nodes are named after the node and contain
    -- the node as a member.
    nodes.name AS name,
    COALESCE(nodes.description, '') as description,
    ARRAY[nodes.name] AS nodes,
    COALESCE(dashboard_node_variables.variables, '{}') as variables,
    COALESCE(dashboard_node_classes.classes, '{}') as classes,
    -- Groups created from Nodes have no parent groups.
    ARRAY[]::varchar[] AS parents
  FROM
    nodes
  LEFT OUTER JOIN
    dashboard_node_variables ON
    nodes.id = dashboard_node_variables.parameterable_id
  LEFT OUTER JOIN
    dashboard_node_classes ON
    nodes.id = dashboard_node_classes.node_id
) AS dashboard_nodes
;
